package com.danitheskunk.skunkworks.gfx.threedee;

import com.danitheskunk.skunkworks.Vec3f;
import com.danitheskunk.skunkworks.Vec3i;
import org.lwjgl.assimp.AIMesh;

import java.util.ArrayList;
import java.util.List;

public class Mesh {
	private List<Vec3i> triangles;
	private List<Vec3f> vertices;

	public Mesh() {
		vertices = new ArrayList<>();
		triangles = new ArrayList<>();
	}

	public static Mesh fromAIMesh(AIMesh aiMesh) {
		var mesh = new Mesh();

		var numVertices = aiMesh.mNumVertices();
		var numFaces = aiMesh.mNumFaces();

		var vertexBuffer = aiMesh.mVertices();
		var faceBuffer = aiMesh.mFaces();

		while(vertexBuffer.remaining() > 0) {
			var vert = vertexBuffer.get();
			mesh.addVertex(new Vec3f(vert.x(), vert.y(), vert.z()));
		}

		while(faceBuffer.remaining() > 0) {
			var face = faceBuffer.get();
			if(face.mNumIndices() != 3) {
				throw new RuntimeException("skunkworks only supports " +
					"triangles currently");
			}
			var indices = face.mIndices();
			mesh.addTriangle(indices.get(0), indices.get(1), indices.get(2));
		}
		return mesh;
	}

	public void addTriangle(int v1, int v2, int v3) {
		triangles.add(new Vec3i(v1, v2, v3));
	}

	public void addVertex(Vec3f vertex) {
		vertices.add(vertex);
	}

	public int getTriangleCount() {
		return triangles.size();
	}

	public int getVertexCount() {
		return vertices.size();
	}

	public Vec3f getVertex(int idx) {
		return vertices.get(idx);
	}

	public Vec3i getTriangle(int idx) {
		return triangles.get(idx);
	}

}
